
#include "galil1800_io.h"

/* Initialize the controller
   Used in PCI probe function
   Return < 0 if errors, 0 otherwise */
int __init galil1800_init_controller_probe_step(struct galil1800_state *s)
{
	const char *RS = "RS\r";
	unsigned long j;
	char buf[GALIL1800_READ_FIFO_SIZE+1];
	int ret, n;

	galil1800_reset_fifo_rw(s);

	/* check if FIFO is really flushed */
	n = galil1800_may_write(s);
	if (0==n || 1==n)
		return -EIO;

	galil1800_send(s, RS, strlen(RS));
	j = jiffies + 5*HZ; /* 5 second timeout */

	n=0;

	/* RS can return ":" or "RS\r:" if echo is enabled */
	do {
		ret=galil1800_recv(s, buf+n, GALIL1800_READ_FIFO_SIZE-n);
		if(!ret) {
			schedule();
			continue;
		}
		n+=ret;
		if (buf[n-1] == ':')
			break;
	}while(time_before(jiffies,j));

	if (!n) {
		printk(KERN_NOTICE PFX "RS command timed out."
				" Performing hardware reset\n");
		galil1800_reset_hardware(s);
	} else if (buf[n-1] != ':') {
		buf[n]='\0';
		printk(KERN_ERR PFX
			"RS command failed! It returned \"%s\"\n", buf);
		return -EIO;
	}

	GALIL_PRINT_DEBUG("device initialized\n");

	return 0;
}


/* This is the top half interrupt handler */
irqreturn_t galil1800_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct galil1800_state *s = dev_id;
	unsigned long flags;
	unsigned char status, t;

	t = inb(GALIL1800_CONTROL_REG(s));
	if(!(t & GALIL1800_IRQ_ENABLE_FLAG) || !(t & GALIL1800_IRQ_STATUS_FLAG))
		return IRQ_NONE; /* shared interrupt raised by someone else */

	status = inb(GALIL1800_IRQ_REG(s));

	spin_lock_irqsave(&s->hwlock, flags);
	t = inb(GALIL1800_CONTROL_REG(s)); /* need to re-read!! */
	outb(t | GALIL1800_IRQ_STATUS_FLAG, GALIL1800_CONTROL_REG(s));
	spin_unlock_irqrestore(&s->hwlock, flags);

	GALIL_PRINT_DEBUG("interrupt handler: status byte is %x\n", status);
	/* This is only to simplify the example!!
	   Generally interrupts are used to inform the driver for a
	   particular event. For example in interrupt driven I/O an
	   interrupt can inform the driver that there is available
	   data for reading, so processes that are blocked in reading
	   can be woken up.
	   Finally, remember that printk() in an interrupt handler
	   must not be used! */

	return IRQ_HANDLED;
}


/* Only the first opened device call this function
   Return < 0 if errors, 0 otherwise */
int galil1800_init_controller_open_step(struct galil1800_state *s)
{
	int ret;

	ret = request_irq(s->irq, galil1800_interrupt,
			SA_SHIRQ, GALIL1800_MODULE_NAME, s);

	if (ret == -EINVAL) {
		printk(KERN_ERR PFX "bad irq number or handler\n");
		return ret;
	}
	if (ret == -EBUSY) {
		printk(KERN_ERR PFX "irq %d busy\n", s->irq);
		return ret;
	}
	if (ret < 0) {
		printk(KERN_ERR PFX "request_irq failed\n");
		return ret;
	}

	galil1800_enable_IRQ(s);

	return 0;
}

/* Only the last released device call this function */
void galil1800_clean_controller_open_step(struct galil1800_state *s)
{
	galil1800_disable_IRQ(s);
	synchronize_irq(s->irq);
	free_irq(s->irq, s);
}
